@@ -2,7 +2,7 @@
 ////**********************************************************************
 ////
 ////  RANDOM FORESTS FOR SURVIVAL, REGRESSION, AND CLASSIFICATION (RF-SRC)
-////  Version 1.2
+////  Version 1.3
 ////
 ////  Copyright 2012, University of Miami
 ////
@@ -78,35 +78,18 @@ unsigned int getTurnOnTraceFlag();
 #endif
 Terminal *makeTerminal() {
   Terminal *parent = (Terminal*) gblock((size_t) sizeof(Terminal));
-  parent -> lmvIndex      = NULL;
-  parent -> lmvIndexSize  = 0;
-  parent -> flmvIndex     = NULL;
-  parent -> flmvIndexSize = 0;;
-  parent -> lmrIndex      = NULL;
-  parent -> lmrIndexSize  = 0;
-  parent -> flmrIndex     = NULL;
-  parent -> flmrIndexSize = 0;;
   parent -> lmiIndex      = NULL;
-  parent -> lmiValue      = NULL;
-  parent -> lmiSizePtr    = NULL;
   parent -> lmiSize       = 0;
-  parent -> flmiIndex     = NULL;
-  parent -> flmiValue     = NULL;
-  parent -> flmiSizePtr   = NULL;
-  parent -> flmiSize      = 0;
-  parent -> leafCount     = 0;
-  parent -> mate          = NULL;
-  parent -> dominant      = 0;
-  parent -> fdominant     = 0;
+  parent -> lmiRaggedIndex  = NULL;
+  parent -> lmiRaggedValue  = NULL;
+  parent -> lmiRaggedSize   = NULL;
+  parent -> nodeID     = 0;
+  parent -> dominant   = FALSE;
   return parent;
 }
 void freeTerminal(Terminal        *parent) {
-  unstackTermLMVIndex(parent);
-  unstackTermFLMVIndex(parent);
-  unstackTermLMRIndex(parent);
-  unstackTermFLMRIndex(parent);
-  unstackTermFLMISizePtr(parent);
-  unstackTermLMISizePtr(parent);
+  unstackTermLMIRagged(parent);
+  unstackTermLMIIndex(parent);
   free_gblock(parent, sizeof(Terminal));
 }
 Node *makeNode(unsigned int xSize) {
@@ -117,6 +100,7 @@ Node *makeNode(unsigned int xSize) {
   for (i = 1; i <= xSize; i++) {
     (parent -> permissibleSplit)[i] = TRUE;
   }
+  parent -> mate               = NULL;
   parent -> left               = NULL; 
   parent -> right              = NULL;  
   parent -> splitFlag            = TRUE;  
@@ -127,7 +111,8 @@ Node *makeNode(unsigned int xSize) {
   parent -> splitValueFactPtr    = NULL;
   parent -> splitStatistic       = NA_REAL;
   parent -> variance             = NA_REAL;
-  parent -> leafCount            = 0;
+  parent -> nodeID               = 0;
+  parent -> orderedNodeID        = 0;
   parent -> depth                = 0;
   parent -> splitDepth           = NULL;
   parent -> pseudoTerminal       = FALSE;
@@ -151,16 +136,16 @@ Node *makeNode(unsigned int xSize) {
   parent -> rfSize               = NULL;
   parent -> multiClassProb       = NULL;
   parent -> membrCount           = 0;
-  parent -> mvSignSize               = 0;
-  parent -> fmvSignSize              = 0;
-  parent -> mvSign               = NULL;
-  parent -> fmvSign              = NULL;
-  parent -> lmvIndex             = NULL;
-  parent -> flmvIndex            = NULL;
-  parent -> lmvIndexAllocSize    = 0;
-  parent -> flmvIndexAllocSize   = 0;
-  parent -> lmvIndexActualSize   = 0;
-  parent -> flmvIndexActualSize  = 0;
+  parent -> mpIndexSize               = 0;
+  parent -> fmpIndexSize              = 0;
+  parent -> mpSign               = NULL;
+  parent -> fmpSign              = NULL;
+  parent -> lmpIndex             = NULL;
+  parent -> flmpIndex            = NULL;
+  parent -> lmpIndexAllocSize    = 0;
+  parent -> flmpIndexAllocSize   = 0;
+  parent -> lmpIndexActualSize   = 0;
+  parent -> flmpIndexActualSize  = 0;
   parent -> lmrIndex             = NULL;
   parent -> flmrIndex            = NULL;
   parent -> lmrIndexAllocSize    = 0;
@@ -188,10 +173,10 @@ void freeNode(Node         *parent) {
       }
     }
   }
-  unstackMVSign(parent);
-  unstackFMVSign(parent);
-  unstackNodeLMVIndex(parent);
-  unstackNodeFLMVIndex(parent);
+  unstackMPSign(parent);
+  unstackFMPSign(parent);
+  unstackNodeLMPIndex(parent);
+  unstackNodeFLMPIndex(parent);
   unstackNodeLMRIndex(parent);
   unstackNodeFLMRIndex(parent);
   if ((parent -> splitParameter) == 0) {
@@ -215,57 +200,57 @@ void freeTerminalNodeSurvivalStructures(Node *terminalNode) {
     unstackCIF(terminalNode);
   }
 }
-void getNodeInfo(Node *leaf) {
+void getNodeInfo(Node *nodePtr) {
   unsigned int i;
-  Rprintf("\nNodeInfo:  %20x", leaf);
+  Rprintf("\nNodeInfo:  %20x", nodePtr);
   Rprintf("\n   LeafCnt   SpltParm  ");
-  Rprintf("\n%10d %10d \n", leaf -> leafCount, leaf -> splitParameter);
-  if (leaf -> splitValueFactSize > 0) {
-    Rprintf("FactorInfo %20x \n", leaf -> splitValueFactPtr);
+  Rprintf("\n%10d %10d \n", nodePtr -> nodeID, nodePtr -> splitParameter);
+  if (nodePtr -> splitValueFactSize > 0) {
+    Rprintf("FactorInfo %20x \n", nodePtr -> splitValueFactPtr);
     Rprintf("0x ");
-    for (i = leaf -> splitValueFactSize; i >= 1; i--) {
-      Rprintf("%8x ", (leaf -> splitValueFactPtr)[i]);
+    for (i = nodePtr -> splitValueFactSize; i >= 1; i--) {
+      Rprintf("%8x ", (nodePtr -> splitValueFactPtr)[i]);
     }
   }
   else {
-    Rprintf(" %12.4f \n", leaf -> splitValueCont);
+    Rprintf(" %12.4f \n", nodePtr -> splitValueCont);
   }
   Rprintf("\nSplit Statistic \n");
-  Rprintf(" %12.4f \n", leaf -> splitStatistic);
+  Rprintf(" %12.4f \n", nodePtr -> splitStatistic);
   Rprintf("\nNode Variance \n");
-  Rprintf(" %12.4f \n", leaf -> variance);
-  if (leaf -> permissibleSplit != NULL) {
+  Rprintf(" %12.4f \n", nodePtr -> variance);
+  if (nodePtr -> permissibleSplit != NULL) {
     Rprintf("\nPermissible Splits \n");
-    for (i=1; i <= leaf -> xSize; i++) {
+    for (i=1; i <= nodePtr -> xSize; i++) {
       Rprintf(" %10d", i);
     }
     Rprintf("\n");
-    for (i=1; i <= leaf -> xSize; i++) {
-      Rprintf(" %10d", (leaf -> permissibleSplit)[i]);
+    for (i=1; i <= nodePtr -> xSize; i++) {
+      Rprintf(" %10d", (nodePtr -> permissibleSplit)[i]);
     }
     Rprintf("\n");
   }
-  Rprintf("\n mvSignSize   = %20d", leaf -> mvSignSize);
-  Rprintf("\n fmvSignSize  = %20d", leaf -> fmvSignSize);
+  Rprintf("\n mpIndexSize   = %20d", nodePtr -> mpIndexSize);
+  Rprintf("\n fmpIndexSize  = %20d", nodePtr -> fmpIndexSize);
   Rprintf("\n");
-  Rprintf("\n mvSign       = %20x", leaf -> mvSign);
-  Rprintf("\n fmvSign      = %20x", leaf -> fmvSign);
+  Rprintf("\n mpSign       = %20x", nodePtr -> mpSign);
+  Rprintf("\n fmpSign      = %20x", nodePtr -> fmpSign);
   Rprintf("\n");
-  Rprintf("\n lmvIndexActualSize        = %20d", leaf -> lmvIndexActualSize);
-  Rprintf("\n flmvIndexActualSize       = %20d", leaf -> flmvIndexActualSize);
-  Rprintf("\n lmvIndexAllocSize         = %20d", leaf -> lmvIndexAllocSize);
-  Rprintf("\n flmvIndexAllocSize        = %20d", leaf -> flmvIndexAllocSize);
+  Rprintf("\n lmpIndexActualSize        = %20d", nodePtr -> lmpIndexActualSize);
+  Rprintf("\n flmpIndexActualSize       = %20d", nodePtr -> flmpIndexActualSize);
+  Rprintf("\n lmpIndexAllocSize         = %20d", nodePtr -> lmpIndexAllocSize);
+  Rprintf("\n flmpIndexAllocSize        = %20d", nodePtr -> flmpIndexAllocSize);
   Rprintf("\n");
-  Rprintf("\n lmvIndex            = %20x", leaf -> lmvIndex);
-  Rprintf("\n flmvIndex           = %20x", leaf -> flmvIndex);
+  Rprintf("\n lmpIndex            = %20x", nodePtr -> lmpIndex);
+  Rprintf("\n flmpIndex           = %20x", nodePtr -> flmpIndex);
   Rprintf("\n");
-  Rprintf("\n lmrIndexActualSize        = %20d", leaf -> lmrIndexActualSize);
-  Rprintf("\n flmrIndexActualSize       = %20d", leaf -> flmrIndexActualSize);
-  Rprintf("\n lmrIndexAllocSize         = %20d", leaf -> lmrIndexAllocSize);
-  Rprintf("\n flmrIndexAllocSize        = %20d", leaf -> flmrIndexAllocSize);
+  Rprintf("\n lmrIndexActualSize        = %20d", nodePtr -> lmrIndexActualSize);
+  Rprintf("\n flmrIndexActualSize       = %20d", nodePtr -> flmrIndexActualSize);
+  Rprintf("\n lmrIndexAllocSize         = %20d", nodePtr -> lmrIndexAllocSize);
+  Rprintf("\n flmrIndexAllocSize        = %20d", nodePtr -> flmrIndexAllocSize);
   Rprintf("\n");
-  Rprintf("\n lmrIndex            = %20x", leaf -> lmrIndex);
-  Rprintf("\n flmrIndex           = %20x", leaf -> flmrIndex);
+  Rprintf("\n lmrIndex            = %20x", nodePtr -> lmrIndex);
+  Rprintf("\n flmrIndex           = %20x", nodePtr -> flmrIndex);
 }
 void setParent(Node *daughter, Node *parent) {
   daughter -> parent = parent;
@@ -644,89 +629,89 @@ void unstackMortality(Node *tNode) {
     }
   }
 }
-void stackMVSign(Node *tNode, unsigned int mvSignSize) {
-  if (tNode -> mvSignSize > 0) {
-    if (tNode -> mvSignSize != mvSignSize) {
+void stackMPSign(Node *tNode, unsigned int mpIndexSize) {
+  if (tNode -> mpIndexSize > 0) {
+    if (tNode -> mpIndexSize != mpIndexSize) {
       Rprintf("\nRF-SRC:  *** ERROR *** ");
-      Rprintf("\nRF-SRC:  mvSignSize has been previously defined:  %10d vs %10d", tNode -> mvSignSize, mvSignSize);
+      Rprintf("\nRF-SRC:  mpIndexSize has been previously defined:  %10d vs %10d", tNode -> mpIndexSize, mpIndexSize);
       Rprintf("\nRF-SRC:  Please Contact Technical Support.");
       error("\nRF-SRC:  The application will now exit.\n");
     }
   }
   else {
-    tNode -> mvSignSize = mvSignSize;
+    tNode -> mpIndexSize = mpIndexSize;
   }
-  tNode -> mvSign = ivector(1, tNode -> mvSignSize);
+  tNode -> mpSign = ivector(1, tNode -> mpIndexSize);
 }
-void unstackMVSign(Node *tNode) {
-  if(tNode -> mvSignSize > 0) {
-    if (tNode -> mvSign != NULL) {
-      free_ivector(tNode -> mvSign, 1, tNode -> mvSignSize);
-      tNode -> mvSign = NULL;
+void unstackMPSign(Node *tNode) {
+  if(tNode -> mpIndexSize > 0) {
+    if (tNode -> mpSign != NULL) {
+      free_ivector(tNode -> mpSign, 1, tNode -> mpIndexSize);
+      tNode -> mpSign = NULL;
     }
   }
 }
-void stackFMVSign(Node *tNode, unsigned int fmvSignSize) {
-  if (tNode -> fmvSignSize > 0) {
-    if (tNode -> fmvSignSize != fmvSignSize) {
+void stackFMPSign(Node *tNode, unsigned int fmpIndexSize) {
+  if (tNode -> fmpIndexSize > 0) {
+    if (tNode -> fmpIndexSize != fmpIndexSize) {
       Rprintf("\nRF-SRC:  *** ERROR *** ");
-      Rprintf("\nRF-SRC:  fmvSignSize has been previously defined:  %10d vs %10d", tNode -> fmvSignSize, fmvSignSize);
+      Rprintf("\nRF-SRC:  fmpIndexSize has been previously defined:  %10d vs %10d", tNode -> fmpIndexSize, fmpIndexSize);
       Rprintf("\nRF-SRC:  Please Contact Technical Support.");
       error("\nRF-SRC:  The application will now exit.\n");
     }
   }
   else {
-    tNode -> fmvSignSize = fmvSignSize;
+    tNode -> fmpIndexSize = fmpIndexSize;
   }
-  tNode -> fmvSign = ivector(1, tNode -> fmvSignSize);
+  tNode -> fmpSign = ivector(1, tNode -> fmpIndexSize);
 }
-void unstackFMVSign(Node *tNode) {
-  if(tNode -> fmvSignSize > 0) {
-    if (tNode -> fmvSign != NULL) {
-      free_ivector(tNode -> fmvSign, 1, tNode -> fmvSignSize);
-      tNode -> fmvSign = NULL;
+void unstackFMPSign(Node *tNode) {
+  if(tNode -> fmpIndexSize > 0) {
+    if (tNode -> fmpSign != NULL) {
+      free_ivector(tNode -> fmpSign, 1, tNode -> fmpIndexSize);
+      tNode -> fmpSign = NULL;
     }
   }
 }
-void stackNodeLMVIndex(Node *tNode, unsigned int size) {
-  if (tNode -> lmvIndexAllocSize > 0) {
+void stackNodeLMPIndex(Node *tNode, unsigned int size) {
+  if (tNode -> lmpIndexAllocSize > 0) {
     Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  lmvIndex has been previously defined:  %10d vs %10d", tNode -> lmvIndexAllocSize, size);
+    Rprintf("\nRF-SRC:  lmpIndex has been previously defined:  %10d vs %10d", tNode -> lmpIndexAllocSize, size);
     Rprintf("\nRF-SRC:  Please Contact Technical Support.");
     error("\nRF-SRC:  The application will now exit.\n");
   }
   else {
-    tNode -> lmvIndexAllocSize = size;
+    tNode -> lmpIndexAllocSize = size;
   }
-  tNode -> lmvIndex = uivector(1, tNode -> lmvIndexAllocSize);
+  tNode -> lmpIndex = uivector(1, tNode -> lmpIndexAllocSize);
 }
-void unstackNodeLMVIndex(Node *tNode) {
-  if(tNode -> lmvIndexAllocSize > 0) {
-    if (tNode -> lmvIndex != NULL) {
-      free_uivector(tNode -> lmvIndex, 1, tNode -> lmvIndexAllocSize);
-      tNode -> lmvIndex = NULL;
-      tNode -> lmvIndexAllocSize = 0;
+void unstackNodeLMPIndex(Node *tNode) {
+  if(tNode -> lmpIndexAllocSize > 0) {
+    if (tNode -> lmpIndex != NULL) {
+      free_uivector(tNode -> lmpIndex, 1, tNode -> lmpIndexAllocSize);
+      tNode -> lmpIndex = NULL;
+      tNode -> lmpIndexAllocSize = 0;
     }
   }
 }
-void stackNodeFLMVIndex(Node *tNode, unsigned int size) {
-  if (tNode -> flmvIndexAllocSize > 0) {
+void stackNodeFLMPIndex(Node *tNode, unsigned int size) {
+  if (tNode -> flmpIndexAllocSize > 0) {
     Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  flmvIndex has been previously defined:  %10d vs %10d", tNode -> flmvIndexAllocSize, size);
+    Rprintf("\nRF-SRC:  flmpIndex has been previously defined:  %10d vs %10d", tNode -> flmpIndexAllocSize, size);
     Rprintf("\nRF-SRC:  Please Contact Technical Support.");
     error("\nRF-SRC:  The application will now exit.\n");
   }
   else {
-    tNode -> flmvIndexAllocSize = size;
+    tNode -> flmpIndexAllocSize = size;
   }
-  tNode -> flmvIndex = uivector(1, tNode -> flmvIndexAllocSize);
+  tNode -> flmpIndex = uivector(1, tNode -> flmpIndexAllocSize);
 }
-void unstackNodeFLMVIndex(Node *tNode) {
-  if(tNode -> flmvIndexAllocSize > 0) {
-    if (tNode -> flmvIndex != NULL) {
-      free_uivector(tNode -> flmvIndex, 1, tNode -> flmvIndexAllocSize);
-      tNode -> flmvIndex = NULL;
-      tNode -> flmvIndexAllocSize = 0;
+void unstackNodeFLMPIndex(Node *tNode) {
+  if(tNode -> flmpIndexAllocSize > 0) {
+    if (tNode -> flmpIndex != NULL) {
+      free_uivector(tNode -> flmpIndex, 1, tNode -> flmpIndexAllocSize);
+      tNode -> flmpIndex = NULL;
+      tNode -> flmpIndexAllocSize = 0;
     }
   }
 }
@@ -772,124 +757,75 @@ void unstackNodeFLMRIndex(Node *tNode) {
     }
   }
 }
-void stackTermLMVIndex(Terminal *tNode, unsigned int size) {
-  if (tNode -> lmvIndexSize > 0) {
-    Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  lmvIndex has been previously defined:  %10d vs %10d", tNode -> lmvIndexSize, size);
-    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-    error("\nRF-SRC:  The application will now exit.\n");
-  }
-  else {
-    tNode -> lmvIndexSize = size;
-  }
-  tNode -> lmvIndex = uivector(1, tNode -> lmvIndexSize);
-}
-void unstackTermLMVIndex(Terminal *tNode) {
-  if(tNode -> lmvIndexSize > 0) {
-    if (tNode -> lmvIndex != NULL) {
-      free_uivector(tNode -> lmvIndex, 1, tNode -> lmvIndexSize);
-      tNode -> lmvIndex = NULL;
-    }
-  }
-}
-void stackTermFLMVIndex(Terminal *tNode, unsigned int size) {
-  if (tNode -> flmvIndexSize > 0) {
-    Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  flmvIndex has been previously defined:  %10d vs %10d", tNode -> flmvIndexSize, size);
-    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-    error("\nRF-SRC:  The application will now exit.\n");
-  }
-  else {
-    tNode -> flmvIndexSize = size;
-  }
-  tNode -> flmvIndex = uivector(1, tNode -> flmvIndexSize);
-}
-void unstackTermFLMVIndex(Terminal *tNode) {
-  if(tNode -> flmvIndexSize > 0) {
-    if (tNode -> flmvIndex != NULL) {
-      free_uivector(tNode -> flmvIndex, 1, tNode -> flmvIndexSize);
-      tNode -> flmvIndex = NULL;
-    }
-  }
-}
-void stackTermLMRIndex(Terminal *tNode, unsigned int size) {
-  if (tNode -> lmrIndexSize > 0) {
-    Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  lmrIndex has been previously defined:  %10d vs %10d", tNode -> lmrIndexSize, size);
-    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-    error("\nRF-SRC:  The application will now exit.\n");
-  }
-  else {
-    tNode -> lmrIndexSize = size;
-  }
-  tNode -> lmrIndex = uivector(1, tNode -> lmrIndexSize);
-}
-void unstackTermLMRIndex(Terminal *tNode) {
-  if(tNode -> lmrIndexSize > 0) {
-    if (tNode -> lmrIndex != NULL) {
-      free_uivector(tNode -> lmrIndex, 1, tNode -> lmrIndexSize);
-      tNode -> lmrIndex = NULL;
-    }
-  }
-}
-void stackTermFLMRIndex(Terminal *tNode, unsigned int size) {
-  if (tNode -> flmrIndexSize > 0) {
-    Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  flmrIndex has been previously defined:  %10d vs %10d", tNode -> flmrIndexSize, size);
-    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-    error("\nRF-SRC:  The application will now exit.\n");
-  }
-  else {
-    tNode -> flmrIndexSize = size;
-  }
-  tNode -> flmrIndex = uivector(1, tNode -> flmrIndexSize);
-}
-void unstackTermFLMRIndex(Terminal *tNode) {
-  if(tNode -> flmrIndexSize > 0) {
-    if (tNode -> flmrIndex != NULL) {
-      free_uivector(tNode -> flmrIndex, 1, tNode -> flmrIndexSize);
-      tNode -> flmrIndex = NULL;
-    }
-  }
-}
-void stackTermFLMISizePtr(Terminal *tNode, unsigned int size) {
-  if (tNode -> flmiSize > 0) {
-    Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  flmiSize has been previously defined:  %10d vs %10d", tNode -> flmiSize, size);
-    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
-    error("\nRF-SRC:  The application will now exit.\n");
-  }
-  else {
-    tNode -> flmiSize = size;
-  }
-  tNode -> flmiSizePtr = uivector(1, tNode -> flmiSize);
-}
-void unstackTermFLMISizePtr(Terminal *tNode) {
-  if(tNode -> flmiSize > 0) {
-    if (tNode -> flmiSizePtr != NULL) {
-      free_uivector(tNode -> flmiSizePtr, 1, tNode -> flmiSize);
-      tNode -> flmiSizePtr = NULL;
-    }
-  }
-}
-void stackTermLMISizePtr(Terminal *tNode, unsigned int size) {
+void stackTermLMIIndex(Terminal *tNode, unsigned int size) {
   if (tNode -> lmiSize > 0) {
     Rprintf("\nRF-SRC:  *** ERROR *** ");
-    Rprintf("\nRF-SRC:  lmiSize has been previously defined:  %10d vs %10d", tNode -> lmiSize, size);
+    Rprintf("\nRF-SRC:  lmiIndex has been previously defined:  %10d vs %10d", tNode -> lmiSize, size);
     Rprintf("\nRF-SRC:  Please Contact Technical Support.");
     error("\nRF-SRC:  The application will now exit.\n");
   }
   else {
     tNode -> lmiSize = size;
   }
-  tNode -> lmiSizePtr = uivector(1, tNode -> lmiSize);
+  tNode -> lmiIndex = uivector(1, tNode -> lmiSize);
+  tNode -> lmiRaggedSize = uivector(1, tNode -> lmiSize);
+  tNode -> lmiRaggedIndex = (unsigned int **) vvector(1, tNode -> lmiSize);
+  tNode -> lmiRaggedValue = (double**) vvector(1, tNode -> lmiSize);
 }
-void unstackTermLMISizePtr(Terminal *tNode) {
+void unstackTermLMIIndex(Terminal *tNode) {
   if(tNode -> lmiSize > 0) {
-    if (tNode -> lmiSizePtr != NULL) {
-      free_uivector(tNode -> lmiSizePtr, 1, tNode -> lmiSize);
-      tNode -> lmiSizePtr = NULL;
+    if (tNode -> lmiIndex != NULL) {
+      free_uivector(tNode -> lmiIndex, 1, tNode -> lmiSize);
+      tNode -> lmiIndex = NULL;
     }
+    if (tNode -> lmiRaggedSize != NULL) {
+      free_uivector(tNode -> lmiRaggedSize, 1, tNode -> lmiSize);
+      tNode -> lmiRaggedSize = NULL;
+    }
+    if (tNode -> lmiRaggedIndex != NULL) {
+      free_vvector(tNode -> lmiRaggedIndex, 1, tNode -> lmiSize);
+      tNode -> lmiRaggedIndex = NULL;
+    }
+    if (tNode -> lmiRaggedValue != NULL) {
+      free_vvector(tNode -> lmiRaggedValue, 1, tNode -> lmiSize);
+      tNode -> lmiRaggedValue = NULL;
+    }
+  }
+}
+void stackTermLMIRagged(Terminal *tNode) {
+  unsigned int i;
+  if (!(tNode -> lmiSize > 0)) {
+    Rprintf("\nRF-SRC:  *** ERROR *** ");
+    Rprintf("\nRF-SRC:  Attempt to allocate lmiRagged with no local missingness:  %20x", tNode);
+    Rprintf("\nRF-SRC:  Please Contact Technical Support.");
+    error("\nRF-SRC:  The application will now exit.\n");
+  }
+  for (i = 1; i <= tNode -> lmiSize; i++) {
+    (tNode -> lmiRaggedIndex)[i] = uivector(1, (tNode -> lmiRaggedSize)[i]);
+    (tNode -> lmiRaggedValue)[i] = dvector(1, (tNode -> lmiRaggedSize)[i]);
+  }
+}
+void unstackTermLMIRagged(Terminal *tNode) {
+  unsigned int i;
+  if(tNode -> lmiSize > 0) {
+    if (tNode -> lmiRaggedIndex != NULL) { 
+      for (i = 1; i <= tNode -> lmiSize; i++) {
+        if ((tNode -> lmiRaggedIndex)[i] != NULL) { 
+          free_uivector((tNode -> lmiRaggedIndex)[i], 1, (tNode -> lmiRaggedSize)[i]);
+          (tNode -> lmiRaggedIndex)[i] = NULL;
+        }
+      }
+    }
+    if (tNode -> lmiRaggedValue != NULL) { 
+      for (i = 1; i <= tNode -> lmiSize; i++) {
+        if ((tNode -> lmiRaggedValue)[i] != NULL) { 
+          free_dvector((tNode -> lmiRaggedValue)[i], 1, (tNode -> lmiRaggedSize)[i]);
+          (tNode -> lmiRaggedValue)[i] = NULL;
+        }
+      }
+    }
+  }
+  else {
   }
 }
 void stackMultiClassProb(Node *tNode, unsigned int rfCount, unsigned int *rfSize) {
